% Copyright 2013, Yi Yu <yuy@comp.nus.edu.sg>
% This program is for Personal Music Video Generation based on Modeling Scene Moods and is free: 
% You can redistribute it and/or modify
% it under the terms of the GNU General Public License.



function feat = getAudioFeat(audio_file)
    % initialization feature extraction
    fs = 22050;                   % sampling rate
    frsize = 2048;                % frame size
    fftlen = frsize;              % fft size

    % get audio signal %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    [p,n,e] = fileparts(audio_file);
    data_fs = 0;
    if strcmpi(e,'.wav')
        [y, data_fs] = wavread(audio_file);
    elseif strcmpi(e,'.mp3')
        [filesize, data_fs] = mp3read(audio_file, 'size');
        if data_fs ~= fs
            if (mod(data_fs,fs) == 0)
                dsfactor = data_fs / fs;
            else
                fprintf(2, '%s, sampling rate %d incompatible with %d\n', ...
                    audio_file, data_fs, fs);
                feat = [];
                return;
            end;
        else
            dsfactor = 1;
        end;
        [y, data_fs] = mp3read(audio_file, [1 fs*60], 1, dsfactor);
    end;
    if data_fs ~= fs
        fprintf(2, 'Error occurs, sampling rate mismatch\n');
        feat = [];
        return;
    end;
        
    % constants for STFT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    fl = round( 100 * (fftlen/fs));
    fu = round(2000 * (fftlen/fs));

    % constants for Pitch %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Index_Freq = zeros(fftlen/2,1);
    Index_Note = zeros(fftlen/2,1);
    for k=1:fftlen/2
        Index_Freq(k) = (k-1) / fftlen * fs;
        Index_Note(k) = round( (12 * log(Index_Freq(k)/440.0) / log(2.0) + 69.0) + 0.5);
    end;
    minNoteIndex =  21;
    maxNoteIndex = 108;

    % constants for MFCC %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    hz2mel = @( hz )( 1127*log(1+hz/700) );     % Hertz to mel warping function
    mel2hz = @( mel )( 700*exp(mel/1127)-700 ); % mel to Hertz warping function

    % Type III DCT matrix routine (see Eq. (5.14) on p.77 of [1])
    dctm = @( N, M )( sqrt(2.0/M) * cos( repmat([0:N-1].',1,M) ...
                                       .* repmat(pi*([1:M]-0.5)/M,N,1) ) );
    % Cepstral lifter routine (see Eq. (5.12) on p.75 of [1])
    ceplifter = @( N, L )( 1+0.5*L*sin(pi*[0:N-1]/L) );
    M = 20;                 % number of filterbank channels 
    C = 19;                 % number of cepstral coefficients
    N = C+1;
    L = 40;                 % cepstral sine lifter parameter
    LF =  100;              % lower frequency limit (Hz)
    HF = 3700;              % upper frequency limit (Hz)
    H = trifbank(M, fftlen/2+1, [LF HF], fs, hz2mel, mel2hz); % size of H is M x K 
    % DCT matrix computation
    DCT = dctm(N, M);
    % Cepstral lifter computation
    lifter = ceplifter(N, L);


    % compute spectral features, with input: y %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    spec = [];
    melspec = [];
    mfccs = [];
    pitch = [];
    for s = 1:frsize/2:max(length(y)-frsize,1);
        yt = zeros(fftlen,1);
        if length(y)<frsize
            yt(1:length(y)) = y;
        else
            yt(1:frsize) = y(s-1+(1:frsize));
        end;
        if mean(abs(yt(1:frsize)))<1e-3
            continue;
        end;
        yt(1:frsize) = hamming(frsize) .* yt(1:frsize);
        yf = abs(fft(yt));
        spec = [spec  yf];

        % Filterbank application to unique part of the magnitude spectrum
        FBE = H * yf(1:fftlen/2+1,:); % FBE( FBE<1.0 ) = 1.0; % apply mel floor
        melspec = [melspec  FBE];

        % Conversion of logFBEs to cepstral coefficients through DCT
        CC =  DCT * log(FBE*2^15);
        % Cepstral liftering gives liftered cepstral coefficients
        CC = diag(lifter) * CC; % ~ HTK's MFCCs
        mfccs = [mfccs  CC];

        % find energy per-pitch
        pf = zeros(fftlen/2,1);
        pf(fl:fu) = yf(fl:fu);
        NoteAmp = zeros(maxNoteIndex,1);
        for j=minNoteIndex:maxNoteIndex
            indicator = (Index_Note==j);
            [maxv, maxi] = max(pf.*indicator);
            NoteAmp(j) = maxv;
        end;
        % summarize energy of harmonic family
        strength = zeros(maxNoteIndex,1);
        for j=minNoteIndex:maxNoteIndex
            if((j+24)<=maxNoteIndex)
                strength(j) = NoteAmp(j) + 0.84*NoteAmp(j+12) + 0.7*NoteAmp(j+19) + 0.6*NoteAmp(j+24);
            elseif((j+19)<=maxNoteIndex)
                strength(j) = NoteAmp(j) + 0.84*NoteAmp(j+12) + 0.7*NoteAmp(j+19);
            elseif((j+12)<=maxNoteIndex)
                strength(j) = NoteAmp(j) + 0.84*NoteAmp(j+12);
            else	
                strength(j) = NoteAmp(j);
            end;
        end;
        [maxv, note] = max(strength);
        pitch = [pitch  note];
    end;

    % specfeat.stft = spec(fl:fu,:);
    specfeat.mfcc = mfccs;
    specfeat.melspec = melspec;
    specfeat.pitch = pitch;
    feat = specfeat;


function [ H, f, c ] = trifbank( M, K, R, fs, h2w, w2h )
    if( nargin~= 6 ), help trifbank; return; end; % very lite input validation

    f_min = 0;          % filter coefficients start at this frequency (Hz)
    f_low = R(1);       % lower cutoff frequency (Hz) for the filterbank 
    f_high = R(2);      % upper cutoff frequency (Hz) for the filterbank 
    f_max = 0.5*fs;     % filter coefficients end at this frequency (Hz)
    f = linspace( f_min, f_max, K ); % frequency range (Hz), size 1xK
    fw = h2w( f );

    % filter cutoff frequencies (Hz) for all filters, size 1x(M+2)
    c = w2h( h2w(f_low)+[0:M+1]*((h2w(f_high)-h2w(f_low))/(M+1)) );
    cw = h2w( c );

    H = zeros( M, K );                  % zero otherwise
    for m = 1:M 
        % implements Eq. (6.141) on page 315 of [1]
        k = f>=c(m)&f<=c(m+1); % up-slope
        H(m,k) = (f(k)-c(m))/(c(m+1)-c(m));
        k = f>=c(m+1)&f<=c(m+2); % down-slope
        H(m,k) = (c(m+2)-f(k))/(c(m+2)-c(m+1));
    end

